[−][src]Struct dyn_cache::local::LocalCache
Holds arbitrary query results which are namespaced by arbitrary scope types. Usually used
through SharedLocalCache::cache_with and SharedLocalCache::gc.
Query types
Note: the types referenced in this documentation are only visible on individual methods, as
LocalCacheis not itself a generic type.
Storage is sharded by the type of the query. The type of a query has three parts:
The query scope is the value which indexes the storage for a particular query type, it has the
bound Scope: 'static + Eq + Hash.
Each Scope corresponds to at most a single Input: 'static
and a single Output: 'static value at any given time.
Reading stored values
See LocalCache::get which accepts borrowed forms of Scope
and Input: Key and Arg respectively. Arg must satisfy PartialEq<Input> to determine
whether to return a stored output.
Garbage Collection
Each time LocalCache::gc is called it removes any values which haven't been
referenced since the prior call.
After each GC, all values still in the cache are marked garbage. They are marked live again when
inserted with LocalCache::store or read with
LocalCache::get.
Implementations
impl LocalCache[src]
pub fn get<'k, Key: ?Sized, Scope, Arg: ?Sized, Input, Output>(
&self,
key: &'k Key,
arg: &Arg
) -> Result<&Output, CacheMiss<'k, Key, Scope, Input, Output>> where
Key: Eq + Hash + ToOwned<Owned = Scope>,
Scope: 'static + Borrow<Key> + Eq + Hash,
Arg: PartialEq<Input> + ToOwned<Owned = Input>,
Input: 'static + Borrow<Arg>,
Output: 'static, [src]
&self,
key: &'k Key,
arg: &Arg
) -> Result<&Output, CacheMiss<'k, Key, Scope, Input, Output>> where
Key: Eq + Hash + ToOwned<Owned = Scope>,
Scope: 'static + Borrow<Key> + Eq + Hash,
Arg: PartialEq<Input> + ToOwned<Owned = Input>,
Input: 'static + Borrow<Arg>,
Output: 'static,
Return a reference to a query's stored output if a result is stored and arg equals the
previously-stored Input. If a reference is returned, the stored input/output
is marked as a root and will not be GC'd the next call.
If no reference is found, a CacheMiss is returned. Call CacheMiss::init to get
a CacheEntry to pass to LocalCache::store.
pub fn store<Key: ?Sized, Scope, Input, Output>(
&mut self,
entry: CacheEntry<'_, Key, Scope, Input, Output>
) where
Key: Eq + Hash + ToOwned<Owned = Scope>,
Scope: 'static + Borrow<Key> + Eq + Hash,
Input: 'static,
Output: 'static, [src]
&mut self,
entry: CacheEntry<'_, Key, Scope, Input, Output>
) where
Key: Eq + Hash + ToOwned<Owned = Scope>,
Scope: 'static + Borrow<Key> + Eq + Hash,
Input: 'static,
Output: 'static,
Stores a fresh CacheEntry whose input/output will not be GC'd at the next call.
Call LocalCache::get to get a CacheMiss and CacheMiss::init to get a
CacheEntry.
pub fn gc(&mut self)[src]
Drop any values which have not been marked alive since the last call to this method.