[−][src]Struct dyn_cache::CacheMiss
The result of a failed attempt to retrieve a value from the cache.
Initialize a full CacheEntry for storage with CacheMiss::init.
use dyn_cache::local::LocalCache; let mut cache = LocalCache::default(); let (scope, arg) = (&'a', &1); let miss = cache.get(scope, arg).expect_err("first access will always be a miss");
Implementations
impl<'k, Key: ?Sized, Scope, Input, Output, H> CacheMiss<'k, Key, Scope, Input, Output, H>[src]
pub fn init<R>(
self,
query: impl FnOnce(&Input) -> (Output, R)
) -> (CacheEntry<'k, Key, Scope, Input, Output, H>, R)[src]
self,
query: impl FnOnce(&Input) -> (Output, R)
) -> (CacheEntry<'k, Key, Scope, Input, Output, H>, R)
Prepare the cache miss to be populated by running query(arg),
returning a separate value. The value returned (R) is typically
derived in some way from the stored Output.
let (entry, result): (_, Vec<usize>) = miss.init(|&n| { let v: Vec<usize> = vec![n; n]; (v.clone(), v) }); cache.store(entry); assert_eq!(result, vec![1usize]);